<?php

namespace Drupal\html_import\Controller;

use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Field\FieldFilteredMarkup;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\node\Entity\Node;

/**
 * Controller to Process HTML files.
 *
 * @package Drupal\html_import.
 */
class NodeCreationBatchOperationController extends ControllerBase {

  /**
   * Process HTML file in batch.
   *
   * @param string $files
   *   The file path.
   * @param string $folder_uri
   *   The file extension.
   * @param string $context
   *   The batch context array.
   */
  public function processHtmlFiles($files, $folder_uri, &$context) {
    // Iterate all the directory files.
    foreach ($files as $file_uri) {
      $node_id = NULL;
      $filename = reset(explode('.', $file_uri->filename));

      // Set node title.
      $node_content['title'] = $filename;
      $node_url = '/' . str_replace(' ', '_', strtolower($node_content['title']));

      // Set node content.
      $node_content['content'] = self::getFileContent($file_uri->uri);

      if (!empty($node_content['content'])) {
        try {
          // Create and update Node.
          $path = \Drupal::service('path.alias_manager')->getPathByAlias($node_url);
          if (preg_match('/node\/(\d+)/', $path, $matches)) {
            $node_id = $matches[1];
          }
          $saved_nid = self::createNode($node_content, $node_id);

          // Success message for node creation.
          if (!empty($saved_nid)) {
            $context['results'][] = $saved_nid;
            \Drupal::logger('html_import')->info(
              t('@nid Node ID created successfully with unpublished status.', ['@nid' => $saved_nid])
            );
          }
        }
        catch (\Throwable $throwable) {
          throw new \Exception($throwable);
        }
      }
      else {
        $message_type = MessengerInterface::TYPE_ERROR;
        \Drupal::logger('html_import')->error(
          t('Uploaded HTML file has invalid content.'));
      }
    }
  }

  /**
   * Get uploaded file content.
   *
   * @param int $file_input
   *   File Fid.
   *
   * @return string
   *   HTML file content.
   */
  public static function getFileContent($file_input) {
    $content = file_get_contents($file_input);
    $node_content = Html::normalize(Xss::filter($content, FieldFilteredMarkup::allowedTags()));
    return $node_content;
  }

  /**
   * Create Node with content type import_pages.
   *
   * @param array $item
   *   Node content.
   * @param int $nid
   *   Node nid.
   *
   * @return int
   *   Node NID.
   */
  public static function createNode(array $item, $nid = NULL) {
    $url = '/' . str_replace(' ', '_', strtolower($item['title']));
    if ($nid == NULL) {
      // Create New Node.
      $node_data['type'] = 'import_pages';
      $node_data['body']['value'] = $item['content'];
      $node_data['body']['format'] = 'full_html';
      $node_data['title'] = $item['title'];
      $node_data['path']['alias'] = $url;
      $node = Node::create($node_data);
      $node->setPublished(FALSE);
    }
    else {
      // Update Node.
      $node = Node::load($nid);
      $node->set('body', $item['content']);
      $node->body->format = 'full_html';
    }
    $node->save();
    return $node->id();

  }

  /**
   * Finish batch to create node from HTML file.
   *
   * @param bool $success
   *   Indicate that the batch API tasks were all completed successfully.
   * @param array $results
   *   An array of all the results that were updated in processHtmlFiles().
   * @param array $operations
   *   A list of the operations that had not been completed by the batch API.
   */
  public static function finishUpMyBatch($success, array $results, array $operations) {
    // The 'success' parameter means no fatal PHP errors were detected. All
    // other error management should be handled using 'results'.
    if ($success) {
      $message = \Drupal::translation()
        ->formatPlural(count($results), 'One node generated .', '@count nodes generated.');
    }
    else {
      $message = $this->t('Batch process finished with an error.');
    }
    \Drupal::messenger()->addMessage($message);
  }

}
